/*
 * YeGods Match Server Manager for SourceMod - www.sourcemod.net
 *
 * coded by dubbeh - www.yegods.net
 *
 * Licensed under the GPLv3
 * 
 */


#include "ygmsm.inc"


public OnPluginStart ()
{
    CreateConVar ("ygmsm_version", PLUGIN_VERSION, "YeGods Match Server Manager version", FCVAR_PLUGIN | FCVAR_SPONLY | FCVAR_REPLICATED | FCVAR_NOTIFY);
    g_cVarEnable = CreateConVar ("ygmsm_enable", "1.0", "Enable the YeGods Match Server Manager", 0, true, 0.0, true, 1.0);
    g_cVarEnableLO3 = CreateConVar ("ygmsm_lo3enable", "1.0", "Use the built in LO3 script", 0, true, 0.0, true, 1.0);
    g_cVarMinPlayersStopTV = CreateConVar ("ygmsm_recordminplayers", "1.0", "Number of minimum players to stop Source TV recording", 0, true, 0.0, true, 10.0);
    g_cVarDemoRecordFolder = CreateConVar ("ygmsm_demorecordfolder", "demos", "The Location relative to the cstrike folder where to save demos");
    g_cVarKnifeRoundStrip = CreateConVar ("ygmsm_kniferoundstrip", "1.0", "Remove all the clients guns & cash in the knife round", 0, true, 0.0, true, 1.0);
    g_cVarMinPlayersBlockQuit = CreateConVar ("ygmsm_minplayersblockquit", "8.0", "Min players that must be on the server to block the quit command", 0, true, 0.0, true, 10.0);
    g_cVarTVEnabled = FindConVar ("tv_enable");
    g_cVarMinRate = FindConVar ("sv_minrate");
    g_cVarMinCmdRate = FindConVar ("sv_mincmdrate");
    g_cVarMinUpdRate = FindConVar ("sv_minupdaterate");

    if ((g_cVarEnable == INVALID_HANDLE) ||
        (g_cVarEnableLO3 == INVALID_HANDLE) ||
        (g_cVarMinPlayersStopTV == INVALID_HANDLE) ||
        (g_cVarDemoRecordFolder == INVALID_HANDLE) ||
        (g_cVarKnifeRoundStrip == INVALID_HANDLE) ||
		(g_cVarMinPlayersBlockQuit == INVALID_HANDLE) ||
		(g_cVarTVEnabled == INVALID_HANDLE) ||
		(g_cVarMinRate == INVALID_HANDLE) ||
		(g_cVarMinCmdRate == INVALID_HANDLE) ||
		(g_cVarMinUpdRate == INVALID_HANDLE))
    {
        SetFailState ("[YGMSM] Error - Unable to create or find a console var.");
        return;
    }

    if (((g_hArrayRecordNames = CreateArray (PLATFORM_MAX_PATH, 0)) == INVALID_HANDLE) || ((g_hArrayIPBans = CreateArray (32, 0)) == INVALID_HANDLE) || ((g_hArrayIDBans = CreateArray (32, 0)) == INVALID_HANDLE))
    {
        SetFailState ("[YGMSM] Error - Unable to create a array");
	}

    /* load translations */
    LoadTranslations ("ygmsm.phrases");

    /* Register client console commands */
    RegConsoleCmd ("say", ChatHookHandler);
    RegConsoleCmd ("say_team", ChatHookHandler);
    RegConsoleCmd ("yg_rates", Command_Rates);

    /* register admin console commands */
    RegAdminCmd ("ygmsm", ShowAdminMenuHandler, ADMFLAG_RCON);

    /* register the server console commands */
    RegServerCmd ("quit", Command_Quit);

    RegisterEventHooks ();
    LoadDemoRecordNamesList ();
    UTIL_SetupOffsets ();
}

public OnPluginEnd ()
{
    ClearArray (g_hArrayIPBans);
    UTIL_FreeHandle (g_hArrayIPBans);
    ClearArray (g_hArrayIDBans);
    UTIL_FreeHandle (g_hArrayIDBans);
    ClearArray (g_hArrayRecordNames);
    UTIL_FreeHandle (g_hArrayRecordNames);

    UnRegisterEventHooks ();
}

public OnMapStart ()
{
    // Setup the map start vars
    g_szDemoRecordName[0] = '\0';
    g_iClientRunningLo3Ko3 = -1;

    for (new i = 0; i <= MAXPLAYERS; i++)
    {
        g_bClientMuted[i] = false;
    }

    g_hAdminMenu = CreateAdminMenu ();
    g_hServerManagementMenu = CreateServerManagementMenu ();
    g_hPlayerManagementMenu = CreatePlayerManagementMenu ();
    g_hMapChangeMenu = CreateListMenuFromFile (g_szMapListFileLoc, Handler_MapChangeMenu, g_szMapSelectionMenuTitle);
    g_hConfigLoadMenu = CreateListMenuFromFile (g_szConfigsFileLoc, Handler_LoadConfigMenu, g_szServerConfigLoadMenuTitle);
    g_hLo3Ko3RRMenu = CreateLo3Ko3RRMenu ();
    g_hServerPasswordMenu = CreateListMenuFromFile (g_szPasswordsFileLoc, Handler_PasswordMenu, g_szServerPasswordMenuTitle);
    g_hRconCommandsMenu = CreateRconCommandListMenu ();
    g_hUnbanTypeMenu = CreateUnbanTypeMenu ();

    ClearArray (g_hArrayIDBans);
    ParseServerBanFiles (BAN_TYPE_ID, g_szBannedUserIDsFileLoc);
    ClearArray (g_hArrayIPBans);
    ParseServerBanFiles (BAN_TYPE_IP, g_szBannedUserIPsFileLoc);

    GetCurrentMap (g_szCurrentMap, sizeof (g_szCurrentMap));
    g_bSourceTVActive = GetConVarBool (g_cVarTVEnabled);
}

public OnConfigsExecuted ()
{
    AutoExecConfig ();
}

public OnMapEnd ()
{
    FreeMenuHandles ();
    ClearArray (g_hArrayIPBans);
    ClearArray (g_hArrayIDBans);
}

public bool:OnClientConnect (client, String:rejectmsg[], maxlen)
{
    g_bClientMuted[client] = false;
    return true;
}

public OnClientDisconnect (client)
{
    g_bClientMuted[client] = false;
}

public Action:ShowAdminMenuHandler (client, args)
{
    if (g_hAdminMenu != INVALID_HANDLE)
        DisplayMenu (g_hAdminMenu, client, MENU_TIME_FOREVER);
    else
        SetFailState ("[YGMSM] Error - Admin menu handle is invalid");

    return Plugin_Handled;
}

public Action:Command_Rates (client, args)
{
    static iMaxClients, iMinRate, iMinCmdRate, iMinUpdRate, i, iUserId, iRate, iCmdRate, iUpdRate, Float:fInterp, iInterpolate;
    decl String:szClientName[MAX_NAME_LENGTH] = "", String:szBuffer[10] = "";

    PrintToConsole (client, "UserID/Name/Rate/CmdRate/UpdRate/Interp/Interpolate");

    iMaxClients = GetMaxClients ();
    iMinRate = GetConVarInt (g_cVarMinRate);
    iMinCmdRate = GetConVarInt (g_cVarMinCmdRate);
    iMinUpdRate = GetConVarInt (g_cVarMinUpdRate);
    i = 1;
    while (i <= iMaxClients)
    {
        if (IsClientConnected (i) && IsClientInGame (i) && !IsFakeClient (i))
        {
            iUserId = GetClientUserId (i);
            GetClientName (i, szClientName, sizeof (szClientName));
            GetClientInfo (i, "rate", szBuffer, sizeof (szBuffer));
            iRate = StringToInt (szBuffer);
            if (iRate < iMinRate)
                iRate = iMinRate;
            GetClientInfo (i, "cl_cmdrate", szBuffer, sizeof (szBuffer));
            iCmdRate = StringToInt (szBuffer);
            if (iCmdRate < iMinCmdRate)
                iCmdRate = iMinCmdRate;
            GetClientInfo (i, "cl_updaterate", szBuffer, sizeof (szBuffer));
            iUpdRate = StringToInt (szBuffer);
            if (iUpdRate < iMinUpdRate)
                iUpdRate = iMinUpdRate;
            GetClientInfo (i, "cl_interp", szBuffer, sizeof (szBuffer));
            fInterp = StringToFloat (szBuffer);
            GetClientInfo (i, "cl_interpolate", szBuffer, sizeof (szBuffer));
            iInterpolate = StringToInt (szBuffer);
            PrintToConsole (client, "%d/%s/%d/%d/%d/%f/%d", iUserId, szClientName, iRate, iCmdRate, iUpdRate, fInterp, iInterpolate);
        }
        i++;
    }

    return Plugin_Handled;
}

